package documenter.control;

import java.util.ArrayList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import documenter.engine.DocGeneratorModel;
import documenter.engine.DocGeneratorView;

/**
 * View class of DocGenerator tool MVC implementation. Implements view interface. 
 * Presents status of the model which is an istance of documenter.engine.DocGeneratorModel.
 * Serves as a access point to the instance of  documenter.control.DocGeneratorGuiControler for users.
 * @author Misio
 *
 */
public class DocGeneratorGui implements DocGeneratorView
{
	private Button generateDocsButton;
	private List list;
	private Label projectNameLabel;
	private Text text;
	private Button addSourcesButton;
	private Button viewDocumentation;
	protected Shell shell;
	private DocGeneratorGuiControler controler;
	private Label sourcesLabel;
	
	/**
	 * Creates instance which will present the status the particular Model instance.
	 * @param model Object which status is to be presented by this view.
	 */
	public DocGeneratorGui(DocGeneratorModel model)
	{
		controler = new DocGeneratorGuiControler(this, model);
	}
	
	/**
	 * 
	 * @return The shell component of the View object.
	 */
	public Shell getShell()
	{
		return shell;
	}
	
	/**
	 * 
	 * @return The button for adding source files
	 */
	public Button getAddSources()
	{
		return addSourcesButton;
	}
	
	/**
	 * 
	 * @return The button for generating documentation files
	 */
	public Button getGenerateDocs()
	{
		return generateDocsButton;
	}
	
	/**
	 * 
	 * @return Text component for entering the documentation project name
	 */
	public Text getText()
	{
		return text;
	}
	
	/**
	 * 
	 * @return The button for opening the system default browser and viewing the generated documentation 
	 */
	public Button getViewDocumentation()
	{
		return viewDocumentation;
	}
	
	/**
	 * Open the View window
	 */
	public void open()
	{
		final Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents()
	{
		shell = new Shell();
		shell.setSize(571, 365);
		shell.setText("Documentation Generator 1.0");
		
		addSourcesButton = new Button(shell, SWT.NONE);
		addSourcesButton.setBounds(10, 20, 168, 24);
		addSourcesButton.setText("Add Source Files");
		addSourcesButton.addSelectionListener(controler);

		projectNameLabel = new Label(shell, SWT.NONE);
		projectNameLabel.setBounds(10, 52, 108, 18);
		projectNameLabel.setText("Project Name:");

		ModifyAdapter adapter = new ModifyAdapter();
		
		text = new Text(shell, SWT.BORDER);
		text.setBounds(10, 74, 170, 25);
		text.addModifyListener(adapter);
		text.setEnabled(false);

		list = new List(shell, SWT.V_SCROLL);
		list.setBounds(198, 41, 354, 212);
		
		generateDocsButton = new Button(shell, SWT.NONE);
		generateDocsButton.setBounds(198, 270, 354, 20);
		generateDocsButton.setEnabled(false);
		generateDocsButton.setText("Gerate Documantation");
		generateDocsButton.addSelectionListener(controler);

		viewDocumentation = new Button(shell, SWT.NONE);
		viewDocumentation.setEnabled(false);
		viewDocumentation.addSelectionListener(controler);
		viewDocumentation.setText("View Documentation");
		viewDocumentation.setBounds(198, 300, 354, 20);

		sourcesLabel = new Label(shell, SWT.NONE);
		sourcesLabel.setText("Selected Sources:");
		sourcesLabel.setBounds(195, 20, 205, 20);
	}
	
	private class ModifyAdapter implements ModifyListener
	{
		public void modifyText(ModifyEvent arg0)
		{
			Text temp;
			temp = (Text) arg0.widget;
			if (temp.getCharCount() == 0)
			{
				generateDocsButton.setEnabled(false);
			}
			else
			{
				generateDocsButton.setEnabled(true);
			}				
		}		
	}

	/**
	 * Adds filenames to the component which displays selected files list
	 */
	public void refreshGeneratorView(ArrayList<String> filenames)
	{
		list.removeAll();
		for(String s : filenames)
		{		
			list.add(s);
		}		
	}

}
