package documenter.control;

import java.io.File;
import java.util.ArrayList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.BrowserControl;
import documenter.engine.DocGeneratorModel;

/**
 * Controler class of DocGenerator tool MVC implementation.
 * Reacts to actions performed on the the View instance by invoking appropriate Model instance methods.
 * Controls appearance of View instance components to ease operability and reduce mis-operability crash possibility.
 * @author Misio
 *
 */
public class DocGeneratorGuiControler implements SelectionListener
{
	private DocGeneratorModel model;
	private DocGeneratorGui gui;
	
	/**
	 * Creates controler instance which operates on particular Model instance, 
	 * responds and controls particular View instance.
	 * @param gui The View instance to react and to control to 
	 * @param model The Model instance which is to be controled
	 */
	public DocGeneratorGuiControler(DocGeneratorGui gui, DocGeneratorModel model)
	{
		this.gui = gui;
		this.model = model;
	}
	
	/**
	 * Reacts to button click events.
	 * For three buttons of gui View class this Controler:
	 * - adds source files to Model instance.
	 * - generates documentation for the given set of source files
	 * - opens the default system browser and displays the generated documentation
	 */
	public void widgetSelected(SelectionEvent arg0)
	{
		try 
		{
			Object source = arg0.getSource();
	        if (source == gui.getAddSources()){
	        	FileDialog fd = new FileDialog(gui.getShell(), SWT.MULTI);
	        	File f = new File("e:\\ontologies");
	        	if(f.exists() && f.isDirectory())
	        	{
	        		fd.setFilterPath("e:\\ontologies");
	        	}
	        	else
	        	{
	        		fd.setFilterPath("c:\\");
	        	}	        		
				String[] filter = new String[1];
				filter[0] = "*.rdf";
				fd.setFilterExtensions(filter);
				fd.open();
				
				String[] fnames = fd.getFileNames();
				ArrayList<String> tempList = new ArrayList<String>(0);
				for(String s : fnames)
				{
					String temppath = fd.getFilterPath() + "\\" + s;
					tempList.add(temppath);
				}
				model.setInput(tempList);
				if (fnames.length > 0)
				{
					gui.getText().setEnabled(true);
				}
	        }
	        if (source == gui.getGenerateDocs()){
	        	DirectoryDialog dd = new DirectoryDialog(gui.getShell(), SWT.SAVE);
	        	File f = new File("e:\\documentation");
	        	if(f.exists() && f.isDirectory())
	        	{
	        		dd.setFilterPath("e:\\documentation");
	        	}
	        	else
	        	{
	        		dd.setFilterPath("c:\\");
	        	}
				String dialogRes = dd.open();
				if(dialogRes != null)
				{
					MessageBox work = new MessageBox(gui.getShell(), SWT.ICON_WORKING);
					work.setText("Working...");
					work.setMessage("Documentation generation in progress. It can take up to few minutes.");
					work.open();
												
					model.setOutpath(dd.getFilterPath());
					model.setProjName(gui.getText().getText());
					model.generateCompleteModel();
									
					MessageBox d = new MessageBox(gui.getShell(), SWT.ICON_INFORMATION);
					d.setText("Information");
					d.setMessage("Documentation has been successfully created. You may view it now.");
					d.open();
					
					gui.getViewDocumentation().setEnabled(true);
				}
	        }
	        if (source == gui.getViewDocumentation())
	        {
	        	if (BrowserControl.isWindowsPlatform())
	        	{
	        		String urlName = model.getDocOutPath() + "\\index.html";
	        		File f = new File(urlName);
	        		if (f.exists())
	        		{
	        			BrowserControl.displayURL(f.toURL().toString());
	        		}
	        	}
	        	else
	        	{
	        		MessageBox errorDialog = new MessageBox(gui.getShell(), SWT.ICON_ERROR);
	    			errorDialog.setText("ERROR");
	    			errorDialog.setMessage("Sorry, supported only on Windows platforms");
	    			errorDialog.open();
	        	}
	        }
		}
		catch (Exception e)
		{
			MessageBox errorDialog = new MessageBox(gui.getShell(), SWT.ICON_ERROR);
			errorDialog.setText("ERROR");
			errorDialog.setMessage(e.getMessage());
			errorDialog.open();		
		}
	}

	/**
	 * Not implemented
	 */
	public void widgetDefaultSelected(SelectionEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

}
